@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2013 The pygit2 contributors
+ * Copyright 2010-2014 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -161,7 +161,7 @@ Branch_remote_name__get__(Branch *self)
         return Error_set(err);
     }
 
-    py_name = to_unicode(c_name, NULL, NULL);
+    py_name = to_unicode_n(c_name, err - 1, NULL, NULL);
     free(c_name);
 
     return py_name;
@@ -253,7 +253,7 @@ Branch_upstream_name__get__(Branch *self)
         return Error_set(err);
     }
 
-    py_name = to_unicode(c_name, NULL, NULL);
+    py_name = to_unicode_n(c_name, err - 1, NULL, NULL);
     free(c_name);
 
     return py_name;
