@@ -71,7 +71,7 @@ Index_dealloc(Index* self)
     PyObject_GC_UnTrack(self);
     Py_XDECREF(self->repo);
     git_index_free(self->index);
-    PyObject_GC_Del(self);
+    Py_TYPE(self)->tp_free(self);
 }
 
 int
@@ -81,7 +81,6 @@ Index_traverse(Index *self, visitproc visit, void *arg)
     return 0;
 }
 
-
 PyDoc_STRVAR(Index_add__doc__,
   "add([path|entry])\n"
   "\n"
@@ -114,6 +113,31 @@ Index_add(Index *self, PyObject *args)
 }
 
 
+PyDoc_STRVAR(Index_add_all__doc__,
+  "add_all([file names|glob pattern])\n"
+  "\n"
+  "Add or update index entries matching files in the working directory.");
+
+PyObject *
+Index_add_all(Index *self, PyObject *pylist)
+{
+    int err;
+    git_strarray pathspec;
+
+    if (get_strarraygit_from_pylist(&pathspec, pylist) < 0)
+        return NULL;
+
+    err = git_index_add_all(self->index, &pathspec, 0, NULL, NULL);
+    git_strarray_free(&pathspec);
+
+    if (err < 0) {
+        Error_set(err);
+        return NULL;
+    }
+
+    Py_RETURN_NONE;
+}
+
 PyDoc_STRVAR(Index_clear__doc__,
   "clear()\n"
   "\n"
@@ -457,6 +481,7 @@ Index_write_tree(Index *self, PyObject *args)
 
 PyMethodDef Index_methods[] = {
     METHOD(Index, add, METH_VARARGS),
+    METHOD(Index, add_all, METH_O),
     METHOD(Index, remove, METH_VARARGS),
     METHOD(Index, clear, METH_NOARGS),
     METHOD(Index, diff_to_workdir, METH_VARARGS),
@@ -536,7 +561,7 @@ void
 IndexIter_dealloc(IndexIter *self)
 {
     Py_CLEAR(self->owner);
-    PyObject_Del(self);
+    Py_TYPE(self)->tp_free(self);
 }
 
 PyObject *
@@ -598,8 +623,9 @@ IndexEntry_init(IndexEntry *self, PyObject *args, PyObject *kwds)
         return -1;
 
     memset(&self->entry, 0, sizeof(struct git_index_entry));
-    if (c_path)
-        self->entry.path = c_path;
+    self->entry.path = strdup(c_path);
+    if (!self->entry.path)
+        return -1;
 
     if (id)
         git_oid_cpy(&self->entry.oid, &id->oid);
@@ -613,6 +639,7 @@ IndexEntry_init(IndexEntry *self, PyObject *args, PyObject *kwds)
 void
 IndexEntry_dealloc(IndexEntry *self)
 {
+    free(self->entry.path);
     PyObject_Del(self);
 }
 
