@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2013 The pygit2 contributors
+ * Copyright 2010-2014 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -33,6 +33,7 @@
 #include <git2.h>
 
 PyObject* Index_add(Index *self, PyObject *args);
+PyObject* Index_add_all(Index *self, PyObject *pylist);
 PyObject* Index_clear(Index *self);
 PyObject* Index_find(Index *self, PyObject *py_path);
 PyObject* Index_read(Index *self, PyObject *args);
@@ -40,7 +41,7 @@ PyObject* Index_write(Index *self);
 PyObject* Index_iter(Index *self);
 PyObject* Index_getitem(Index *self, PyObject *value);
 PyObject* Index_read_tree(Index *self, PyObject *value);
-PyObject* Index_write_tree(Index *self);
+PyObject* Index_write_tree(Index *self, PyObject *args);
 Py_ssize_t Index_len(Index *self);
 int Index_setitem(Index *self, PyObject *key, PyObject *value);
 
