@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2013 The pygit2 contributors
+ * Copyright 2010-2014 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -27,13 +27,24 @@
 
 #define PY_SSIZE_T_CLEAN
 #include <Python.h>
-#include <osdefs.h>
+
+/* Pypy does not provide this header */
+#ifndef PYPY_VERSION
+# include <osdefs.h>
+#endif
+
 #include <git2.h>
 #include "error.h"
 #include "types.h"
 #include "utils.h"
 #include "repository.h"
 #include "oid.h"
+#include "options.h"
+
+/* FIXME: This is for pypy */
+#ifndef MAXPATHLEN
+# define MAXPATHLEN 1024
+#endif
 
 extern PyObject *GitError;
 
@@ -56,12 +67,15 @@ extern PyTypeObject IndexEntryType;
 extern PyTypeObject IndexIterType;
 extern PyTypeObject WalkerType;
 extern PyTypeObject ConfigType;
+extern PyTypeObject ConfigIterType;
 extern PyTypeObject ReferenceType;
 extern PyTypeObject RefLogIterType;
 extern PyTypeObject RefLogEntryType;
 extern PyTypeObject BranchType;
 extern PyTypeObject SignatureType;
 extern PyTypeObject RemoteType;
+extern PyTypeObject RefspecType;
+extern PyTypeObject TransferProgressType;
 extern PyTypeObject NoteType;
 extern PyTypeObject NoteIterType;
 extern PyTypeObject BlameType;
@@ -102,6 +116,14 @@ init_repository(PyObject *self, PyObject *args) {
     Py_RETURN_NONE;
 };
 
+static int
+credentials_cb(git_cred **out, const char *url, const char *username_from_url, unsigned int allowed_types, void *data)
+{
+    PyObject *credentials = (PyObject *) data;
+
+    return callable_to_credentials(out, url, username_from_url, allowed_types, credentials);
+}
+
 PyDoc_STRVAR(clone_repository__doc__,
     "clone_repository(url, path, bare, remote_name, checkout_branch)\n"
     "\n"
@@ -130,11 +152,12 @@ clone_repository(PyObject *self, PyObject *args) {
     const char *path;
     unsigned int bare, ignore_cert_errors;
     const char *remote_name, *checkout_branch;
+    PyObject *credentials = NULL;
     int err;
     git_clone_options opts = GIT_CLONE_OPTIONS_INIT;
 
-    if (!PyArg_ParseTuple(args, "zzIIzz",
-                          &url, &path, &bare, &ignore_cert_errors, &remote_name, &checkout_branch))
+    if (!PyArg_ParseTuple(args, "zzIIzzO",
+                          &url, &path, &bare, &ignore_cert_errors, &remote_name, &checkout_branch, &credentials))
         return NULL;
 
     opts.bare = bare;
@@ -142,6 +165,11 @@ clone_repository(PyObject *self, PyObject *args) {
     opts.remote_name = remote_name;
     opts.checkout_branch = checkout_branch;
 
+    if (credentials != Py_None) {
+	    opts.remote_callbacks.credentials = credentials_cb;
+	    opts.remote_callbacks.payload = credentials;
+    }
+
     err = git_clone(&repo, url, path, &opts);
     if (err < 0)
         return Error_set(err);
@@ -231,6 +259,7 @@ PyMethodDef module_methods[] = {
      discover_repository__doc__},
     {"hashfile", hashfile, METH_VARARGS, hashfile__doc__},
     {"hash", hash, METH_VARARGS, hash__doc__},
+    {"option", option, METH_VARARGS, option__doc__},
     {NULL}
 };
 
@@ -246,6 +275,12 @@ moduleinit(PyObject* m)
     ADD_CONSTANT_INT(m, LIBGIT2_VER_REVISION)
     ADD_CONSTANT_STR(m, LIBGIT2_VERSION)
 
+    /* libgit2 options */
+    ADD_CONSTANT_INT(m, GIT_OPT_GET_SEARCH_PATH);
+    ADD_CONSTANT_INT(m, GIT_OPT_SET_SEARCH_PATH);
+    ADD_CONSTANT_INT(m, GIT_OPT_GET_MWINDOW_SIZE);
+    ADD_CONSTANT_INT(m, GIT_OPT_SET_MWINDOW_SIZE);
+
     /* Errors */
     GitError = PyErr_NewException("_pygit2.GitError", NULL, NULL);
     Py_INCREF(GitError);
@@ -299,7 +334,7 @@ moduleinit(PyObject* m)
     /*
      * Log
      */
-    INIT_TYPE(WalkerType, NULL, PyType_GenericNew)
+    INIT_TYPE(WalkerType, NULL, NULL)
     ADD_TYPE(m, Walker);
     ADD_CONSTANT_INT(m, GIT_SORT_NONE)
     ADD_CONSTANT_INT(m, GIT_SORT_TOPOLOGICAL)
@@ -316,7 +351,7 @@ moduleinit(PyObject* m)
     /*
      * References
      */
-    INIT_TYPE(ReferenceType, NULL, PyType_GenericNew)
+    INIT_TYPE(ReferenceType, NULL, NULL)
     INIT_TYPE(RefLogEntryType, NULL, NULL)
     INIT_TYPE(RefLogIterType, NULL, NULL)
     INIT_TYPE(NoteType, NULL, NULL)
@@ -332,7 +367,7 @@ moduleinit(PyObject* m)
     /*
      * Branches
      */
-    INIT_TYPE(BranchType, &ReferenceType, PyType_GenericNew);
+    INIT_TYPE(BranchType, &ReferenceType, NULL);
     ADD_TYPE(m, Branch)
     ADD_CONSTANT_INT(m, GIT_BRANCH_LOCAL)
     ADD_CONSTANT_INT(m, GIT_BRANCH_REMOTE)
@@ -341,7 +376,7 @@ moduleinit(PyObject* m)
      * Index & Working copy
      */
     INIT_TYPE(IndexType, NULL, PyType_GenericNew)
-    INIT_TYPE(IndexEntryType, NULL, NULL)
+    INIT_TYPE(IndexEntryType, NULL, PyType_GenericNew)
     INIT_TYPE(IndexIterType, NULL, NULL)
     ADD_TYPE(m, Index)
     ADD_TYPE(m, IndexEntry)
@@ -411,12 +446,29 @@ moduleinit(PyObject* m)
     ADD_CONSTANT_INT(m, GIT_DIFF_FIND_AND_BREAK_REWRITES)
 
     /* Config */
+    ADD_CONSTANT_INT(m, GIT_CONFIG_LEVEL_LOCAL);
+    ADD_CONSTANT_INT(m, GIT_CONFIG_LEVEL_GLOBAL);
+    ADD_CONSTANT_INT(m, GIT_CONFIG_LEVEL_XDG);
+    ADD_CONSTANT_INT(m, GIT_CONFIG_LEVEL_SYSTEM);
+
     INIT_TYPE(ConfigType, NULL, PyType_GenericNew)
+    INIT_TYPE(ConfigIterType, NULL, NULL)
     ADD_TYPE(m, Config)
+    ADD_TYPE(m, ConfigIter)
 
     /* Remotes */
     INIT_TYPE(RemoteType, NULL, NULL)
+    INIT_TYPE(RefspecType, NULL, NULL)
+    INIT_TYPE(TransferProgressType, NULL, NULL)
     ADD_TYPE(m, Remote)
+    ADD_TYPE(m, Refspec)
+    ADD_TYPE(m, TransferProgress)
+    /* Direction for the refspec */
+    ADD_CONSTANT_INT(m, GIT_DIRECTION_FETCH)
+    ADD_CONSTANT_INT(m, GIT_DIRECTION_PUSH)
+    /* Credential types */
+    ADD_CONSTANT_INT(m, GIT_CREDTYPE_USERPASS_PLAINTEXT)
+    ADD_CONSTANT_INT(m, GIT_CREDTYPE_SSH_KEY)
 
     /* Blame */
     INIT_TYPE(BlameType, NULL, NULL)
